<?php
/**
 * Daisycon abstract block
 *
 * @category  Daisycon
 * @package   Daisycon_Affiliatemarketing
 * @copyright Copyright (c) 2018 Daisycon B.V.
 * @author    Daisycon B.V. <support@daisycon.com>
 *
 * @method getOrderIds
 * @method setOrderIds
 */
abstract class Daisycon_Affiliatemarketing_Block_Abstract extends Mage_Core_Block_Template
{
    /**
     * The current order
     *
     * @var \Mage_Sales_Model_Order|null
     */
    protected $_order = null;

    /**
     * Get the daisycon affiliatemarketing config
     *
     * @return \Daisycon_Affiliatemarketing_Model_Config
     */
    protected function _getConfig()
    {
        return Mage::getSingleton('affiliatemarketing/config');
    }

    /**
     * Get the current order
     *
     * @return \Mage_Sales_Model_Order|null
     */
    protected function _getOrder()
    {
        if (null === $this->_order) {
            $orderIds = $this->getOrderIds();

            if (false === empty($orderIds) && true === is_array($orderIds)) {
                $collection = Mage::getModel('sales/order')->getCollection();
                $collection->addFilter('entity_id', array('in' => $orderIds));

                foreach ($collection as $order) {
                    // Break on first saved order
                    $id = $order->getId();
                    if (false === empty($id)) {
                        $this->_order = $order;
                        break;
                    }
                }
            }
        }

        return $this->_order;
    }

    /**
     * Get the configured program id
     *
     * @return int|null
     */
    public function getProgramId()
    {
        return $this->_getConfig()->getGeneralProgramId();
    }

    /**
     * Get the tracking domain
     *
     * @return string|null
     */
    public function getTrackingSegment()
    {
        return $this->_getConfig()->getTrackingSegment();
    }

}
