<?php
/**
 * Daisycon Affiliatemarketing extension data helper
 *
 * @category  Daisycon
 * @package   Daisycon_Affiliatemarketing
 * @copyright Copyright (c) 2018 Daisycon B.V.
 * @author    Daisycon B.V. <support@daisycon.com>
 */
class Daisycon_Affiliatemarketing_Helper_Data extends Mage_Core_Helper_Abstract
{
    const GENDER_MALE   = 'M';
    const GENDER_FEMALE = 'F';

    /**
     * Get the daisycon affiliatemarketing config
     *
     * @return Daisycon_Affiliatemarketing_Model_Config
     */
    protected function _getConfig()
    {
        return Mage::getSingleton('affiliatemarketing/config');
    }

    /**
     * Get product value by attribute
     *
     * @param Mage_Catalog_Model_Product $product
     * @param string $attribute
     * @return string|int
     */
    public function getProductValue(Mage_Catalog_Model_Product $product, $attribute)
    {
        $model = $product->getResource()->getAttribute($attribute);
        if (true === is_callable(array($model, 'getFrontend'))) {
            return $model->getFrontend()->getValue($product);
        } else {
            return $product->getData($attribute);
        }
    }

    /**
     * Get the id of the product category
     *
     * @param Mage_Catalog_Model_Product $product
     * @return int|null
     */
    public function getCategoryId(Mage_Catalog_Model_Product $product)
    {
        $categoryIds = $product->getCategoryIds();
        if (true === empty($categoryIds)) {
            return null;
        }

        return reset($categoryIds);
    }

    /**
     * Gender id
     *
     * @param int $genderId
     * @return string
     */
    public function getGender($genderId)
    {
        switch ((int)$genderId)
        {
            case 1:
                return self::GENDER_MALE;

            case 2:
                return self::GENDER_FEMALE;

            default:
                return '';
        }
    }

    /**
     * Load account data form the Daisycon API
     *
     * @param string $username
     * @param string $password
     * @throws Mage_Core_Exception
     * @return array
     */
    public function loadAccountData($username, $password)
    {
        $connection = new Daisycon_Affiliatemarketing_Model_Api_Connection(
            array(
                'username' => $username,
                'password' => $password,
            )
        );

        $advertiserModel = new Daisycon_Affiliatemarketing_Model_Api_Resource_Advertiser($connection);
        $programModel = new Daisycon_Affiliatemarketing_Model_Api_Resource_Advertiser_Program($connection);

        $tree = array();
        $programIds = array();
        try {
            $advertisers = $advertiserModel->all(array('order' => 'name'));

            if (true === empty($advertisers)) {
                return array();
            }

            usort($advertisers, array($this, 'sort'));

            foreach ($advertisers as $advertiser) {
                $node = array_intersect_key($advertiser->getAttributes(), array_flip(array('id', 'name')));
                $programs = $programModel->all(
                    array(
                        'advertiser_id' => $advertiser->getPrimaryKeyValue(),
                        'order'         => 'name',
                    )
                );

                if (true === empty($programs)) {
                    continue;
                }

                $node['programs'] = array();

                usort($programs, array($this, 'sort'));

                foreach ($programs as $program) {
                    $data = array_intersect_key(
                        $program->getAttributes(),
                        array_flip(array('id', 'name', 'tracking_segment'))
                    );

                    if (true === empty($data)) {
                        continue;
                    }

                    $programIds[] = $data['id'];
                    $node['programs'][] = $data;
                }

                $tree[] = $node;
            }
        } catch (Exception $exception) {
            Mage::throwException($exception->getMessage());
        }

        return $tree;
    }

    /**
     * Compare models by the name attribute
     *
     * @param Daisycon_Affiliatemarketing_Model_Api_Resource_Abstract $a
     * @param Daisycon_Affiliatemarketing_Model_Api_Resource_Abstract $b
     * @return int
     */
    protected static function sort($a, $b)
    {
        if ($a->name === $b->name) {
            return 0;
        }

        return strcasecmp($a->name, $b->name);
    }
}
