<?php
/**
 * Daisycon api http client
 *
 * @category  Daisycon
 * @package   Daisycon_Affiliatemarketing
 * @copyright Copyright (c) 2018 Daisycon B.V.
 * @author    Daisycon B.V. <support@daisycon.com>
 */
class Daisycon_Affiliatemarketing_Model_Api_Client
{
    const ERROR_UNSUPPORTED_PROTOCOL = 1;
    const ERROR_REQUEST_FAILED       = 2;

    /**
     * Send a request
     *
     * @param Daisycon_Affiliatemarketing_Model_Api_Request $request
     * @throws Mage_Core_Exception
     * @throws RuntimeException
     * @return Daisycon_Affiliatemarketing_Model_Api_Response
     */
    public function send(Daisycon_Affiliatemarketing_Model_Api_Request $request)
    {
        $timeout = 100;

        $url = $request->getUrl();

        // verify that the URL uses a supported protocol.
        if (false === strpos($url, 'http://') && false === strpos($url, 'https://')) {
            throw new Mage_Core_Exception('Unsupported protocol', self::ERROR_UNSUPPORTED_PROTOCOL);
        }

        // set URL and other appropriate options
        $options = array(
            CURLOPT_CUSTOMREQUEST => $request->getMethod(),
        );

        if (false !== strpos($url, 'https://')) {
            $options[CURLOPT_SSL_VERIFYHOST] = false;
            $options[CURLOPT_SSL_VERIFYPEER] = false;
        }

        $options[CURLOPT_RETURNTRANSFER] = 1;
        $options[CURLOPT_TIMEOUT] = $timeout;
        $options[CURLOPT_HEADER] = true;

        $curl = new Varien_Http_Adapter_Curl();
        $curl->setOptions($options);
        $curl->write(
            $request->getMethod(),
            $request->getUrl(),
            null, $this->_httpEncodeHeaders($request->getHeaders()),
            $request->getBody()
        );

        // Execute the request.
        $response = $curl->read();
        $succeeded = 0 === $curl->getErrno() ? true : false;

        // If Communication was not successful set error result, otherwise
        if (false === $succeeded) {
            throw new RuntimeException('Request failed', self::ERROR_REQUEST_FAILED);
        }

        $statusCode = $curl->getInfo(CURLINFO_HTTP_CODE);

        // close cURL resource, and free up system resources
        $curl->close();

        list($headers, $body) = explode("\r\n\r\n", $response, 2);

        return new Daisycon_Affiliatemarketing_Model_Api_Response(
            $body,
            $this->_httpParseHeaders($headers),
            $statusCode
        );
    }

    /**
     * Encode the http headers
     *
     * @param array $headers
     * @return array
     */
    protected function _httpEncodeHeaders(array $headers)
    {
        $rawHeaders = array();
        foreach ($headers as $header => $value) {
            if (false === is_string($header)) {
                $rawHeaders[] = $value;
                continue;
            }

            $rawHeaders[] = null !== $value ? $header . ': ' . $value : $header;
        }

        return $rawHeaders;
    }

    /**
     * Parse http headers
     *
     * @param string
     * @return array
     */
    protected function _httpParseHeaders($rawHeaders)
    {
        if (true === function_exists('http_parse_headers')) {
            return http_parse_headers($rawHeaders);
        }

        $headers = array();
        $key = '';

        foreach (explode("\n", $rawHeaders) as $line) {
            $line = explode(':', $line, 2);
            if (false === isset($line[1])) {
                if (false === isset($headers[$line[0]]))
                    $headers[$line[0]] = true === isset($line[1]) ? trim($line[1]) : '';
                elseif (true === is_array($headers[$line[0]])) {
                    $headers[$line[0]] = array_merge($headers[$line[0]], array(trim($line[1])));
                } else {
                    $headers[$line[0]] = array_merge(array($headers[$line[0]]), array(trim($line[1])));
                }

                $key = $line[0];
            } else {
                if ("\t" === substr($line[0], 0, 1)) {
                    $headers[$key] .= "\r\n\t" . trim($line[0]);
                } elseif (true === empty($key)) {
                    $headers[0] = trim($line[0]);
                }
            }
        }

        return $headers;
    }
}
