<?php
/**
 * Daisycon affiliatemarketing plugin config
 *
 * @category  Daisycon
 * @package   Daisycon_Affiliatemarketing
 * @copyright Copyright (c) 2018 Daisycon B.V.
 * @author    Daisycon B.V. <support@daisycon.com>
 */
class Daisycon_Affiliatemarketing_Model_Config
{
    const CONFIG_DATA_ACCOUNT_USERNAME                     = 'affiliatemarketing/account/username';
    const CONFIG_DATA_GENERAL_LCC_ENABLED                  = 'affiliatemarketing/general/lcc_enabled';
    const CONFIG_DATA_GENERAL_LCC_URL_PARAM                = 'affiliatemarketing/general/lcc_url_param';
    const CONFIG_DATA_GENERAL_PROGRAM_ID                   = 'affiliatemarketing/general/program_id';
    const CONFIG_DATA_GENERAL_ADVERTISER_PROGRAMS          = 'affiliatemarketing/general/advertiser_programs';
    const CONFIG_DATA_COMPENSATION_VAT                     = 'affiliatemarketing/compensation/vat';
    const CONFIG_DATA_COMPENSATION_FIELD                   = 'affiliatemarketing/compensation/field';
    const CONFIG_DATA_COMPENSATION_FIELD_PRODUCT_ATTRIBUTE = 'affiliatemarketing/compensation/field_product_attribute';
    const CONFIG_DATA_TRACKING_E1                          = 'affiliatemarketing/tracking/e1';
    const CONFIG_DATA_TRACKING_E1_PRODUCT_ATTRIBUTE        = 'affiliatemarketing/tracking/e1_product_attribute';
    const CONFIG_DATA_TRACKING_E2                          = 'affiliatemarketing/tracking/e2';
    const CONFIG_DATA_TRACKING_E2_PRODUCT_ATTRIBUTE        = 'affiliatemarketing/tracking/e2_product_attribute';
    const CONFIG_DATA_TRACKING_E3                          = 'affiliatemarketing/tracking/e3';
    const CONFIG_DATA_TRACKING_E3_PRODUCT_ATTRIBUTE        = 'affiliatemarketing/tracking/e3_product_attribute';
    const CONFIG_DATA_TRACKING_E4                          = 'affiliatemarketing/tracking/e4';
    const CONFIG_DATA_TRACKING_E4_PRODUCT_ATTRIBUTE        = 'affiliatemarketing/tracking/e4_product_attribute';
    const CONFIG_DATA_TRACKING_E5                          = 'affiliatemarketing/tracking/e5';
    const CONFIG_DATA_TRACKING_E5_PRODUCT_ATTRIBUTE        = 'affiliatemarketing/tracking/e5_product_attribute';

    /**
     * Get api service url
     *
     * @return string
     */
    public function getApiServiceUrl()
    {
        return (string)Mage::getConfig()->getNode('global/daisycon/affiliatemarketing/service/url');
    }

    /**
     * Get the account username
     * 
     * @param mixed $store
     * @return string
     */
    public function getAccountUsername($store = null)
    {
        return trim(Mage::getStoreConfig(self::CONFIG_DATA_ACCOUNT_USERNAME, $store));
    }
    
    /**
     * Get the program id
     *
     * @param mixed $store
     * @return string
     */
    public function getGeneralProgramId($store = null)
    {
        return trim(Mage::getStoreConfig(self::CONFIG_DATA_GENERAL_PROGRAM_ID, $store));
    }

    /**
     * Get the advertiser program tree
     *
     * @param mixed $store
     * @return array|null
     */
    public function getGeneralAdvertiserPrograms($store = null)
    {
        try {
            return Mage::helper('core')->jsonDecode(
                Mage::getStoreConfig(self::CONFIG_DATA_GENERAL_ADVERTISER_PROGRAMS, $store)
            );
        } catch (Zend_Json_Exception $exception) {
            return null;
        }
    }

    /**
     * Set the advertiser programs
     *
     * @param array $value
     * @param mixed $store
     * @return $this
     */
    public function setAdvertiserPrograms(array $value, $store = null)
    {
        $scope = $this->_getScope($store);

        $value = Mage::helper('core')->jsonEncode($value);
        Mage::getModel('core/config')->saveConfig(
            self::CONFIG_DATA_GENERAL_ADVERTISER_PROGRAMS,
            $value,
            $scope['name'],
            $scope['id']
        )->cleanCache();

        return $this;
    }

    /**
     * Get the compensation field
     *
     * @param mixed $store
     * @return string
     */
    public function getCompensationField($store = null)
    {
        return trim(Mage::getStoreConfig(self::CONFIG_DATA_COMPENSATION_FIELD, $store));
    }

    /**
     * Get the compensation field product attribute
     *
     * @param mixed $store
     * @return string
     */
    public function getCompensationFieldProductAttribute($store = null)
    {
        return trim(Mage::getStoreConfig(self::CONFIG_DATA_COMPENSATION_FIELD_PRODUCT_ATTRIBUTE, $store));
    }

    /**
     * Get compensation field vat enabled/disabled
     *
     * @param mixed $store
     * @return bool
     */
    public function getCompensationVat($store = null)
    {
        return (bool)Mage::getStoreConfig(self::CONFIG_DATA_COMPENSATION_VAT, $store);
    }

    /**
     * Get the tracking segment domain
     *
     * @param mixed $store
     * @return string|null
     */
    public function getTrackingSegment($store = null)
    {
        $programId = (int)$this->getGeneralProgramId($store);
        $tree = $this->getGeneralAdvertiserPrograms($store);
        foreach ($tree as $advertiser) {
            if (true === empty($advertiser['programs'])) {
                continue;
            }

            foreach ($advertiser['programs'] as $program) {
                if ($programId === (int)$program['id']) {
                    return $program['tracking_segment'];
                }
            }
        }

        return null;
    }

    /**
     * Is tracking active
     *
     * @param mixed $store
     * @return bool
     */
    public function isTrackingActive($store = null)
    {
        return (int)$this->getGeneralProgramId($store) > 0;
    }

    /**
     * Is lcc active
     *
     * @param mixed $store
     * @return bool
     */
    public function isLccActive($store = null)
    {
        return Mage::getStoreConfigFlag(self::CONFIG_DATA_GENERAL_LCC_ENABLED, $store);
    }

    /**
     * Retrieve LCC network parameter name.
     *
     * @param mixed $store
     * @return string
     */
    public function getLccNetworkParameterName($store = null)
    {
        $paramName = Mage::getStoreConfig(self::CONFIG_DATA_GENERAL_LCC_URL_PARAM, $store);

        return $paramName
            ? $paramName
            : 'network';
    }

    /**
     * Get extra tracking parameters
     *
     * @param mixed $store
     * @return array
     */
    public function getTrackingExtra($store = null)
    {
        return array(
            1 => Mage::getStoreConfig(self::CONFIG_DATA_TRACKING_E1, $store),
            2 => Mage::getStoreConfig(self::CONFIG_DATA_TRACKING_E2, $store),
            3 => Mage::getStoreConfig(self::CONFIG_DATA_TRACKING_E3, $store),
            4 => Mage::getStoreConfig(self::CONFIG_DATA_TRACKING_E4, $store),
            5 => Mage::getStoreConfig(self::CONFIG_DATA_TRACKING_E5, $store),
        );
    }

    /**
     * Get extra tracking parameters product attributes
     *
     * @param mixed $store
     * @return array
     */
    public function getTrackingExtraProductAttribute($store = null)
    {
        return array(
            1 => Mage::getStoreConfig(self::CONFIG_DATA_TRACKING_E1_PRODUCT_ATTRIBUTE, $store),
            2 => Mage::getStoreConfig(self::CONFIG_DATA_TRACKING_E2_PRODUCT_ATTRIBUTE, $store),
            3 => Mage::getStoreConfig(self::CONFIG_DATA_TRACKING_E3_PRODUCT_ATTRIBUTE, $store),
            4 => Mage::getStoreConfig(self::CONFIG_DATA_TRACKING_E4_PRODUCT_ATTRIBUTE, $store),
            5 => Mage::getStoreConfig(self::CONFIG_DATA_TRACKING_E5_PRODUCT_ATTRIBUTE, $store),
        );
    }

    /**
     * Get the scope and scope id by store
     * 
     * @param mixed $store
     * @return array
     */
    protected function _getScope($store = null)
    {
        $code = Mage::getSingleton('adminhtml/config_data')->getStore($store);
        $scope = 'default';
        $scopeId = 0;
        // Store level
        if (false === empty($code)) {
            $scope = 'store';
            $scopeId = Mage::getModel('core/store')->load($code)->getId();
        } else {
            $code = Mage::getSingleton('adminhtml/config_data')->getWebsite();
            // Website level
            if (false === empty($code)) {
                $websiteId = Mage::getModel('core/website')->load($code)->getId();
                $scope = 'website';
                $scopeId = Mage::app()->getWebsite($websiteId)->getDefaultStore()->getId();
            }
        }

        return array(
            'id'   => $scopeId,
            'name' => $scope,
        );
    }
}
