<?php
/**
 * Program id field selection
 *
 * @category  Daisycon
 * @package   Daisycon_Affiliatemarketing
 * @copyright Copyright (c) 2018 Daisycon B.V.
 * @author    Daisycon B.V. <support@daisycon.com>
 */
class Daisycon_Affiliatemarketing_Model_System_Config_Source_General_Program
    extends Daisycon_Affiliatemarketing_Model_System_Config_Source_Abstract
{
    const PAYMENT_METHOD    = 'payment_method';
    const SHIPPING_METHOD   = 'shipping_method';
    const CATEGORY_ID       = 'category_id';
    const PRODUCT_ATTRIBUTE = 'product_attribute';

    /**
     * Get the daisycon affiliatemarketing config
     *
     * @return \Daisycon_Affiliatemarketing_Model_Config
     */
    protected function _getConfig()
    {
        return Mage::getSingleton('affiliatemarketing/config');
    }

    /**
     * Return all options
     *
     * @return array|null
     */
    public function toOptionArray()
    {
        if (null === $this->_options) {
            $tree = $this->_getConfig()->getGeneralAdvertiserPrograms();

            if (false === empty($tree)) {
                $this->_options = array();
                foreach ($tree as $advertiser) {
                    if (true === empty($advertiser['programs'])) {
                        continue;
                    }

                    $options = array();

                    foreach ($advertiser['programs'] as $program) {
                        if (true === empty($program)) {
                            continue;
                        }

                        $options[] = array(
                            'label' => false === empty($program['name']) ?
                                $program['name'] . ' (' . $program['id'] . ')':
                                Mage::helper('affiliatemarketing')->__('Unknown') . ' (' . $program['id'] . ')',
                            'value' => $program['id'],
                        );
                    }

                    $this->_options[] = array(
                        'label' => $advertiser['name'] . ' (' . $advertiser['id'] . ')',
                        'value' => $options,
                    );
                }

                array_unshift(
                    $this->_options,
                    array(
                        'value' => '',
                        'label' => Mage::helper('affiliatemarketing')->__('-- Select --')
                    )
                );
            }
        }

        return $this->_options;
    }

}
